// $Id: quotient2.C,v 1.1 2001/07/14 18:07:56 dvermeir Exp $
// Example program p. 244.

// The conversion string -> double is put in a separate
// reusable abstraction, making the program more
// readable than the version in quotient1.C.

#include <iostream>
#include <string>
#include <stdexcept> // for standard exception classes
#include <stdlib.h> // for strtod(char*,char**)

// quotient: write quotient of arg1 and arg2 on stdout.

static const string	USAGE("usage: quotient number number");
static const string	DIVISION_BY_ZERO("cannot divide by 0");

// This function returns the double represented by s; it throws
// a range_error exception if s does not represent a double.
double
cstr2double(const char* s) throw(range_error) {
static const string FORMAT_ERROR("not a number (says cstr2double)");

char*	end;
double	d(strtod(s,&end));
if (s==end)
  throw range_error(string(s)+": "+FORMAT_ERROR);
return d;
}

int
main(int argc,char* argv[]) { 
try {
  if (argc!=3) 
    throw runtime_error(USAGE);
  double	a1(cstr2double(argv[1]));
  double	a2(cstr2double(argv[2]));
  if (a2==0)
    throw runtime_error(DIVISION_BY_ZERO);
  cout << a1/a2 << endl;
  return 0;
  }
catch (exception& e) { // must be reference to preserve e's ``real'' type
  cerr << e.what() << endl;
  return 1; // error return
  }
}
